#include <SFML/OpenGL.hpp>
#include <stdio.h>
#include "utils.h"
using namespace std;

#include "ball.h"

Ball::Ball(unsigned int _team, unsigned int _player, Player *_throwPlayer, Vector _dir, Vector _pos, unsigned int numBounces, unsigned long _startTime) {
	dir = _dir;
	pos = _pos;
	old_pos = pos;
	team = _team;
	player = _player;
	bouncesLeft = numBounces;
	startTime = _startTime;
	testSelfCol = false;
	throwPlayer = _throwPlayer;
}

void Ball::draw() {

	glPushMatrix();

	glTranslatef(pos.x, pos.y, 0);

	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2d(1,0);	glVertex2i(BALLRADIUS, -BALLRADIUS);
		glTexCoord2d(1,1);	glVertex2i(BALLRADIUS, BALLRADIUS);
		glTexCoord2d(0,0);	glVertex2i(-BALLRADIUS,-BALLRADIUS);
		glTexCoord2d(0,1);	glVertex2i(-BALLRADIUS,BALLRADIUS);

	glEnd();

	glPopMatrix();
}

void Ball::clock(unsigned long gTime) {
	last_pos = pos;
	pos.x = old_pos.x + (gTime - startTime) * dir.x * 0.1;
	pos.y = old_pos.y + (gTime - startTime) * dir.y * 0.1;
}

void Ball::intersect(Wall obj, unsigned long gTime) {

	float i[3];
	float j[3];

	i[0] = obj.pt2.x - obj.pt1.x;
	i[1] = obj.pt2.y - obj.pt1.y;
	i[2] = 0;

	float length = sqrt(i[0] * i[0] + i[1] * i[1]);


	float W[3];
	W[0] = pos.x - obj.pt1.x;
	W[1] = pos.y - obj.pt1.y;
	W[2] = 0;

	float distVec[3];

	CROSS(distVec, i, W);

	float lengthI;
	lengthI = fabs(distVec[2])/length;

	if(lengthI < BALLRADIUS) {


	Vector pt1, pt2;
	int axis = (fabs(i[0]) > fabs(i[1])) ? 0:1;
	if(axis == 0) {
		pt1 = (obj.pt1.x > obj.pt2.x) ? obj.pt2:obj.pt1;
		pt2 = (obj.pt1.x > obj.pt2.x) ? obj.pt1:obj.pt2;
	} else {
		pt1 = (obj.pt1.y > obj.pt2.y) ? obj.pt2:obj.pt1;
		pt2 = (obj.pt1.y > obj.pt2.y) ? obj.pt1:obj.pt2;
	}



		if(axis == 0) {
			if( !((pos.x > pt1.x) && (pos.x < pt2.x))) {
				return;
			}
		} else {
			if( !((pos.y > pt1.y) && (pos.y < pt2.y))) {
				return;
			}
		}


		float temp[] = {0,0,1};

		CROSS(j,i,temp);

		float Vi, Vj;

		DOT(Vi, dir.x, dir.y, 0, i[0], i[1], i[2]);
		DOT(Vj, dir.x, dir.y, 0, j[0], j[1], j[2]);

		Vj = -1 * Vj;

		DOT(dir.x, Vi, Vj, 0, i[0], i[1], i[2]);
		DOT(dir.y, Vi, Vj, 0, j[0], j[1], j[2]);

		length = sqrt(dir.x*dir.x + dir.y*dir.y);
		dir.x = dir.x / length;
		dir.y = dir.y / length;

		pos.x = pos.x + (BALLRADIUS-lengthI+0.01f) * dir.x;
		pos.y = pos.y + (BALLRADIUS-lengthI+0.01f) * dir.y;

		old_pos.x = pos.x;
		old_pos.y = pos.y;
		startTime = gTime;
		bouncesLeft--;
		testSelfCol = true;

	}
}

void Ball::intersect(Player *obj, unsigned long gTime) {

	Vector wPos;

	if(obj->notPlaying == true) return;
	if(obj->drawState == DHIT) return;

	switch(obj->teamID) {
		case 0: wPos.x = (obj->drawPos.x*RES) - ZONEWIDTH + RES/2;
				wPos.y = (obj->drawPos.y*RES) + (BACKDIST - ZONEHEIGH) + RES/2;
				break;
		case 1: wPos.x = (-BACKDIST + ZONEHEIGH) - (obj->drawPos.y*RES) - RES/2;
				wPos.y = (-ZONEWIDTH) + (obj->drawPos.x*RES) + RES/2;
				break;
		case 2: wPos.x = (ZONEWIDTH) - (obj->drawPos.x*RES) - RES/2;
				wPos.y = (-BACKDIST + ZONEHEIGH) - (obj->drawPos.y*RES) - RES/2;
				break;
		case 3: wPos.x = (BACKDIST - ZONEHEIGH) + (obj->drawPos.y*RES) + RES/2;
				wPos.y = (ZONEWIDTH) - (obj->drawPos.x*RES) - RES/2;
				break;
	};


	float dst = (wPos.x - this->pos.x)*(wPos.x - this->pos.x) + (wPos.y - this->pos.y)*(wPos.y - this->pos.y);
	if(dst < (BALLRADIUS+PLAYERHEIGHT/2)*(BALLRADIUS+PLAYERHEIGHT/2)) {

		if(obj->teamID == team){
			if(obj->id == player) {
				if(testSelfCol == false) {
					return;
				}
			}
		}


		dir.x = -1 * dir.x;
		dir.y = -1 * dir.y;

		float hitAngle;

		if(fabs(dir.y) > fabs(dir.x)) {
			DOT(hitAngle,0,1,0,dir.x, dir.y,0);
			hitAngle = acos(hitAngle);
		} else {
			DOT(hitAngle,1,0,0,dir.x, dir.y,0);
			hitAngle = acos(hitAngle);
			hitAngle = hitAngle + 3.14/2;
		}
		hitAngle = hitAngle * 180/3.14;

		while(hitAngle < 0) {
			hitAngle = hitAngle + 360;
		}

		while(hitAngle > 360) {
			hitAngle = hitAngle - 360;
		}

		float diffAngle = fabs(hitAngle-obj->angle);
		float diffAngleI = fabs(hitAngle+360-obj->angle);

		diffAngle = min(diffAngle, diffAngleI);

		float localHitAngle = hitAngle-obj->offsetA;
		float localAngle = obj->angle-obj->offsetA;
		while(localHitAngle < 0) {
			localHitAngle = localHitAngle + 360;
		}
		while(localHitAngle > 360) {
			localHitAngle = localHitAngle - 360;
		}
		while(localAngle < 0) {
			localAngle = localAngle + 360;
		}
		while(localAngle > 360) {
			localAngle = localAngle - 360;
		}

		string playerState;

//DMOVE, DSTAND, DISOUT, DTHROUGH1, DTHROUGH2, DDEFFEND, DHIT

		switch(obj->drawState) {
			case DMOVE: playerState = string("Move"); break;
			case DSTAND: playerState = string("Stand"); break;
			case DISOUT: playerState = string("Out"); break;
			case DTHROUGH1:
			case DTHROUGH2: playerState = string("Throw"); break;
			case DDEFFEND: playerState = string("Defend"); break;
			case DHIT: playerState = string("hit"); break;
			default: playerState = string("????");
		};

		if((diffAngle < obj->holdAngle/2) && (obj->drawState==DDEFFEND) && (obj->holdPower >= bouncesLeft)) {
			// player caought ball
			obj->holdPower -= bouncesLeft;
			if(obj->teamID != team) {
				throwPlayer->drawState = DISOUT;
				throwPlayer->notPlaying = true;
				*obj->teamScore = *obj->teamScore + 1;
				printf("Team: %s Player: %d caught Team: %s Player: %d ball\n", obj->teamName.c_str(), obj->id+1, throwPlayer->teamName.c_str(), throwPlayer->id+1);
				printf("   |--> Ball:\n");
				printf("   |      |--> Direction: (%f, %f)\n", this->dir.x, this->dir.y);
				printf("   |      |--> Hit Angle: %f\n", localHitAngle);
				printf("   |      |--> Power: %d\n", bouncesLeft);
				printf("   |--> Team: %s Player: %d\n", obj->teamName.c_str(), obj->id+1);
				printf("   |      |--> Score(+): %d\n", *obj->teamScore);
				printf("   |      |--> State: %s\n", playerState.c_str());
				printf("   |      |--> Previous Hold Power: %d\n", obj->holdPower + bouncesLeft);
				printf("   |      |--> Remaining Hold Power: %d\n", obj->holdPower);
				printf("   |      |--> Angle: %f\n", localAngle);
				printf("   |      |--> Hold Angle: (+/-)%f\n", obj->holdAngle/2);
				printf("   |      |--> Diff Angle & Hit Angle: %f\n", diffAngle);
				printf("   |--> Team: %s Player: %d\n", throwPlayer->teamName.c_str(), throwPlayer->id+1);
				printf("          |--> Score(=): %d\n", *throwPlayer->teamScore);
				printf("          |--> Player is out!\n\n");

			} else {
				printf("Team: %s Player: %d caught his own teams ball\n", obj->teamName.c_str(), obj->id+1);
				printf("   |--> Ball:\n");
				printf("   |      |--> Direction: (%f, %f)\n", this->dir.x, this->dir.y);
				printf("   |      |--> Hit Angle: %f\n", localHitAngle);
				printf("   |      |--> Power: %d\n", bouncesLeft);
				printf("   |--> Team: %s Player: %d\n", obj->teamName.c_str(), obj->id+1);
				printf("          |--> Score(=): %d\n", *obj->teamScore);
				printf("          |--> State: %s\n", playerState.c_str());
				printf("          |--> Previous Hold Power: %d\n", obj->holdPower + bouncesLeft);
				printf("          |--> Remaining Hold Power: %d\n", obj->holdPower);
				printf("          |--> Angle: %f\n", localAngle);
				printf("          |--> Hold Angle: (+/-)%f\n", obj->holdAngle/2);
				printf("          |--> Diff Angle & Hit Angle: %f\n\n", diffAngle);
			}
		} else {
			// player hit
			obj->isHit = true;
			obj->drawState = DHIT;
			obj->nextStepTimeStamp = gTime + 1000;

			if(obj->teamID!=team) {
				*throwPlayer->teamScore = *throwPlayer->teamScore + 1;
				printf("Team: %s Player: %d hit Team: %s Player: %d ball\n", throwPlayer->teamName.c_str(), throwPlayer->id+1, obj->teamName.c_str(), obj->id+1);
				printf("   |--> Ball:\n");
				printf("   |      |--> Direction: (%f, %f)\n", this->dir.x, this->dir.y);
				printf("   |      |--> Hit Angle: %f\n", localHitAngle);
				printf("   |      |--> Power: %d\n", bouncesLeft);
				printf("   |--> Team: %s Player: %d\n", throwPlayer->teamName.c_str(), throwPlayer->id+1);
				printf("   |      |--> Score(+): %d\n", *throwPlayer->teamScore);
				printf("   |--> Team: %s Player: %d\n", obj->teamName.c_str(), obj->id+1);
				printf("          |--> Score(=): %d\n", *obj->teamScore);
				printf("          |--> State: %s\n", playerState.c_str());
				printf("          |--> Previous Hold Power: %d\n", obj->holdPower);
				printf("          |--> Angle: %f\n", localAngle);
				printf("          |--> Hold Angle: (+/-)%f\n", obj->holdAngle/2);
				printf("          |--> Diff Angle & Hit Angle: %f\n", diffAngle);
				printf("          |--> Player is out!\n\n");
			} else {
				*throwPlayer->teamScore = *throwPlayer->teamScore - 1;
				printf("Team: %s Player: %d hit his own team\n", throwPlayer->teamName.c_str(), throwPlayer->id+1);
				printf("   |--> Ball:\n");
				printf("   |      |--> Direction: (%f, %f)\n", this->dir.x, this->dir.y);
				printf("   |      |--> Hit Angle: %f\n", localHitAngle);
				printf("   |      |--> Power: %d\n", bouncesLeft);
				printf("   |--> Team: %s Player: %d\n", obj->teamName.c_str(), obj->id+1);
				printf("          |--> Score(-): %d\n", *obj->teamScore);
				printf("          |--> State: %s\n", playerState.c_str());
				printf("          |--> Previous Hold Power: %d\n", obj->holdPower);
				printf("          |--> Angle: %f\n", localAngle);
				printf("          |--> Hold Angle: (+/-)%f\n", obj->holdAngle/2);
				printf("          |--> Diff Angle & Hit Angle: %f\n", diffAngle);
				printf("          |--> Player is out!\n\n");
			}
		}

		bouncesLeft = -1;

	}
}

void Ball::intersect (Ball *obj, unsigned long gTime) {
	float dst = (obj->pos.x - this->pos.x)*(obj->pos.x - this->pos.x) + (obj->pos.y - this->pos.y)*(obj->pos.y - this->pos.y);
	if(dst < (BALLRADIUS*2)*(BALLRADIUS*2)) {

 		testSelfCol = true;
		obj->testSelfCol = true;


		Vector b2, b1;
		Vector b2v, b1v;

		b2 = obj->pos;
		b1 = this->pos;
		b2v = obj->dir;
		b1v = this->dir;

		float b1m, b2m;
		b1m = this->bouncesLeft+1;
		b2m = obj->bouncesLeft+1;

		Vector v_n;

		v_n.x = b2.x-b1.x;
		v_n.y = b2.y-b1.y;

		float hyp = sqrt((v_n.x*v_n.x) + (v_n.y*v_n.y));
		Vector v_un;
		v_un.x = v_n.x/hyp;
		v_un.y = v_n.y/hyp;

		Vector v_ut;
		v_ut.x = -v_un.y;
		v_ut.y = v_un.x;

		float v1n = v_un.x * b1v.x + v_un.y * b1v.y;
		float v1tPrime = v_ut.x * b1v.x + v_ut.y * b1v.y;
		float v2n = v_un.x * b2v.x + v_un.y * b2v.y;
		float v2tPrime = v_ut.x * b2v.x + v_ut.y * b2v.y;

		float v1nPrime = (v1n * (b1m - b2m) + 2 * b2m * v2n)/(b1m + b2m);
		float v2nPrime = (v2n * (b2m - b1m) + 2 * b1m * v1n)/(b1m+b2m);

		Vector v_v1nPrime, v_v1tPrime, v_v2nPrime, v_v2tPrime;
		v_v1nPrime.x = v_un.x*v1nPrime;
		v_v1nPrime.y = v_un.y * v1nPrime;

		v_v1tPrime.x = v_ut.x*v1tPrime;
		v_v1tPrime.y = v_ut.y *v1tPrime;

		v_v2nPrime.x = v_un.x*v2nPrime;
		v_v2nPrime.y = v_un.y*v2nPrime;

		v_v2tPrime.x = v_ut.x*v2tPrime;
		v_v2tPrime.y = v_ut.y*v2tPrime;

		b1v.x = v_v1nPrime.x + v_v1tPrime.x;
		b1v.y = v_v1nPrime.y + v_v1tPrime.y;
		b2v.x = v_v2nPrime.x + v_v2tPrime.x;
		b2v.y = v_v2nPrime.y + v_v2tPrime.y;

		float length = sqrt(b1v.x*b1v.x + b1v.y*b1v.y);
		b1v.x = b1v.x/length;
		b1v.y = b1v.y/length;

		length = sqrt(b2v.x*b2v.x + b2v.y*b2v.y);
		b2v.x = b2v.x/length;
		b2v.y = b2v.y/length;



		obj->dir = b2v;
		this->dir = b1v;

		float i[3];
		i[0] = this->pos.x - obj->pos.x;
		i[1] = this->pos.y - obj->pos.y;
		i[2] = 0.0f;
		/*

		//Conservation of momentum
		Vector newObjV;
		newObjV.x = (obj->dir.x * (obj->bouncesLeft - this->bouncesLeft) + this->dir.x*2*this->bouncesLeft) / (obj->bouncesLeft + this->bouncesLeft);
		newObjV.y = (obj->dir.y * (obj->bouncesLeft - this->bouncesLeft) + this->dir.y*2*this->bouncesLeft) / (obj->bouncesLeft + this->bouncesLeft);

		this->dir.x = (obj->dir.x * (2 * obj->bouncesLeft) + this->dir.x*(obj->bouncesLeft - this->bouncesLeft))/(obj->bouncesLeft + this->bouncesLeft);
		this->dir.y = (obj->dir.y * (2 * obj->bouncesLeft) + this->dir.y*(obj->bouncesLeft - this->bouncesLeft))/(obj->bouncesLeft + this->bouncesLeft);

		obj->dir.x = newObjV.x;
		obj->dir.y = newObjV.y;

		float length = sqrt((this->dir.x * this->dir.x) + (this->dir.y * this->dir.y));
		this->dir.x = this->dir.x / length;
		this->dir.y = this->dir.y / length;

		length = sqrt((obj->dir.x * obj->dir.x) + (obj->dir.y * obj->dir.y));
		obj->dir.x = obj->dir.x / length;
		obj->dir.y = obj->dir.y / length;

		*/

		this->pos.x = obj->pos.x + 1.1001 * i[0];
		this->pos.y = obj->pos.y + 1.1001 * i[1];

		obj->old_pos = obj->pos;
		this->old_pos = this->pos;

		startTime = gTime;
		obj->startTime = gTime;
		bouncesLeft--;
		obj->bouncesLeft--;
	}
}

void Ball::intersectWalls(unsigned long gTime) {
	if(fabs(pos.x) > (RES*GRID/2)+1) {
		bouncesLeft = -1;
	}
	if(fabs(pos.y) > (RES*GRID/2)+1) {
		bouncesLeft = -1;
	}

	/*if(fabs(pos.x) > RES*GRID/2) {
		dir.x = -dir.x;
		pos.x = (dir.x > 0) ? -RES*GRID/2: RES*GRID/2;

		old_pos.x = pos.x;
		old_pos.y = pos.y;
		startTime = gTime;
		bouncesLeft--;
		testSelfCol = true;
	}


	if(fabs(pos.y) > RES*GRID/2) {
		dir.y = -dir.y;
		pos.y = (dir.y > 0) ? -RES*GRID/2: RES*GRID/2;

		old_pos.x = pos.x;
		old_pos.y = pos.y;
		startTime = gTime;
		bouncesLeft--;
		testSelfCol = true;
	}
	*/

}
