#include <stdio.h>
#include "utils.h"
using namespace std;
#include "Game.h"

GAME::GAME() {
	start = false;
	score[0] = 0;
	score[1] = 0;
	score[2] = 0;
	score[3] = 0;

	numBallsLeft[0] = MAXTHROW;
	numBallsLeft[1] = MAXTHROW;
	numBallsLeft[2] = MAXTHROW;
	numBallsLeft[3] = MAXTHROW;

	gameOver = false;
}

void GAME::set(ISUCode *_teamA, ISUCode *_teamB, ISUCode *_teamC, ISUCode *_teamD) {
		teams[0] = _teamA;
		teams[1] = _teamB;
		teams[2] = _teamC;
		teams[3] = _teamD;
		start = false;


		for(unsigned int i = 0; i < 4; i++) {
			for(unsigned int j = 0; j < 4; j++) {
				players[i][j] = new Player(teams[i], j, i, &teamBalls[i], &score[i]);
			}
		}
}

void GAME::setWalls(char *fileName) {

	FILE *pFile;

	pFile = fopen(fileName, "r");
	if(pFile == NULL) {
		printf("Error: can not find file: %s\n", fileName);
		myExit(-1);
	}

	while(!feof(pFile)) {
		Wall temp;
		if(fscanf(pFile, " ( %f , %f ) ( %f , %f ) ", &temp.pt1.x, &temp.pt1.y, &temp.pt2.x, &temp.pt2.y) != 4) {
			printf("Error parsing wall data file: format is (#.#, #.#) (#.#, #.#)\n");
			myExit(-1);
		}
		gameWalls.push_back(temp);
	}


}

void GAME::startGame() {
	start = true;
}

void GAME::clock(){


		static unsigned long gTime = 0;

 		if(start == false) return;
		if(gameOver == true) return;


		for(unsigned int i = 0; i < 4; i++) {
			for(unsigned int j = 0; j < 4; j++) {
				players[i][j]->clock(gTime, &numBallsLeft[i]);
			}
			for(unsigned int j = 0; j < teamBalls[i].size(); j++) {
				teamBalls[i].at(j).clock(gTime);
			}
		}

		// do intersection tests
		for(unsigned int i = 0; i < 4; i++) {
			for(unsigned int j = 0; j < teamBalls[i].size(); j++) {
				// for all the balls in play

				// test with own teams balls
				for(unsigned int jj = j+1; jj < teamBalls[i].size(); jj++) {
					teamBalls[i].at(j).intersect(&teamBalls[i].at(jj),gTime);
				}

				// test agenst all other teams balls
				for(unsigned int ii = i+1; ii < 4; ii++) {
					for(unsigned int jj = 0; jj < teamBalls[ii].size(); jj++) {
						teamBalls[i].at(j).intersect(&teamBalls[ii].at(jj),gTime);
					}
				}

				// test agenst all players
				for(unsigned int k = 0; k < 4; k++) {
					for(unsigned int m = 0; m < 4; m++) {
						teamBalls[i].at(j).intersect(players[k][m], gTime);
					}
				}

				//test agenst walls
				for(unsigned int k = 0; k < gameWalls.size(); k++) {
					teamBalls[i].at(j).intersect(gameWalls.at(k), gTime);
				}

				// test agenst perimiter
				teamBalls[i].at(j).intersectWalls(gTime);


			}
		}

		// remove dead balls
		for(unsigned int i = 0; i < 4; i++) {
			unsigned int numTest = teamBalls[i].size();
			for(int j = 0; j < numTest; j++) {
				if(teamBalls[i].at(j).bouncesLeft < 0) {
					teamBalls[i].erase(teamBalls[i].begin() + j);
					j--;
					numTest--;
				}
			}
		}

		gTime += 30;

		gameOver = true;

		unsigned int numLeft = 0;
		for(unsigned int i = 0; i < 4; i++) {
			for(unsigned int j = 0; j < 4; j++) {
				if(players[i][j]->notPlaying == true) {
					numLeft++;
				}
			}
		}

		if(numLeft != 16) {
			// check if game is over
			for(unsigned int i = 0; i < 4; i++) {
				if(teamBalls[i].size() > 0) {
					gameOver = false;
					return;
				}
				for(unsigned int j = 0; j < 4; j++) {
					if((players[i][j]->outofCode == false)) {
						if(players[i][j]->notPlaying == false) {
							gameOver = false;
							return;
						}
					}
				}
			}
		}
		highScore = -1000000;
		for(unsigned int i = 0; i < 4; i++) {
			highScore = (score[i] > highScore) ? score[i]:highScore;
		}

		printf("Results\n");
		printf("----------------------------------------------------------------\n");
		printf("  %s: %d\n", teams[0]->teamName.c_str(), score[0]);
		printf("  %s: %d\n", teams[1]->teamName.c_str(), score[1]);
		printf("  %s: %d\n", teams[2]->teamName.c_str(), score[2]);
		printf("  %s: %d\n", teams[3]->teamName.c_str(), score[3]);
		printf("----------------------------------------------------------------\n\n");
}
