#include <GL/gl.h>
#include <GL/glut.h>
#include "utils.h"

using namespace std;

#include "Player.h"

Player::Player() {
		id = 0;
		notPlaying = false;
		outofCode = false;
		isHit = false;
		nextStepTimeStamp = 0;
		drawState = DSTAND;
		lineNumber = -1;
		angle = 0;
}

Player::Player(ISUCode *teamCode, unsigned int playerID, unsigned int _teamID, vector<Ball> *_teamBalls, int *_teamScore) {
		id = playerID;
		teamID = _teamID;
		drawPos = teamCode->startPos[playerID];
		startPos = drawPos;
		notPlaying = false;
		outofCode = false;
		isHit = false;
		nextStepTimeStamp = 0;
		drawState = DSTAND;
		lineNumber = -1;
		teamScore = _teamScore;
		teamName = teamCode->teamName;


		playerCode = &teamCode->player[playerID];
		moveVelocity = teamCode->moveVelocity[playerID];
		strength = teamCode->strength[playerID];
		holdAngle = teamCode->holdAngle[playerID];
		holdPower = teamCode->holdPower[playerID];
		throughTime = teamCode->throughTime[playerID];
		teamBalls = _teamBalls;

		switch(_teamID) {
			case 0: offsetX=-ZONEWIDTH;
					offsetY=BACKDIST - ZONEHEIGH;
					offsetA = 0.0f;
					break;

			case 1:
					offsetX=-BACKDIST + ZONEHEIGH;
					offsetY=-ZONEWIDTH;
					offsetA = 270.0f;
					break;

			case 2: ZONEWIDTH;
					offsetX=ZONEWIDTH;
					offsetY=-BACKDIST + ZONEHEIGH;
					offsetA = 180.0f;
					break;

			case 3: offsetX=BACKDIST - ZONEHEIGH;
					offsetY=ZONEWIDTH;
					offsetA = 90.0f;
					break;

		};

		angle = offsetA;


}

void Player::draw(bool gameOver, bool winTeam) {
		glPushMatrix();

		glTranslatef(drawPos.x*RES, drawPos.y*RES, 0);
		//glTranslatef(RES/2, RES/2,0);

		for(unsigned int i =0; i < teamID; i++) {
			glRotatef(-90,0,0,1);
		}

		glPushMatrix();
		if((drawState == DDEFFEND) && (gameOver == false)) {
			switch(teamID) {
				case 0: glColor3d(0.302f, .996f, 0.462745f); break;
				case 1: glColor3d(1.0f, 1.0f, 1.0f); break;
				case 2: glColor3d(0.282353f, 0.513725f, 0.823529f); break;
				case 3: glColor3d(1.0f, 0.941f, 0.0f); break;
				default: glColor3d(1.0f, 1.0f, 1.0f);
			};

			glRotatef(angle,0,0,-1);
			glRotatef(holdAngle/2,0,0,1);
			glBegin(GL_LINES);
				glVertex2i(0,0);
				glVertex2i(0,PLAYERHEIGHT-10);
			glEnd();

			glRotatef(-holdAngle,0,0,1);
			glBegin(GL_LINES);
				glVertex2i(0,0);
				glVertex2i(0,PLAYERHEIGHT-10);
			glEnd();
		}

		glPopMatrix();


		float sRight, sLeft, sTop, sBottom;
		getState(&sRight, &sLeft, &sTop, &sBottom, gameOver, winTeam);


		glColor3d(1.0f, 1.0f, 1.0f);
		glEnable(GL_TEXTURE_2D);

		glBegin(GL_TRIANGLE_STRIP);
			if(((drawState == DISOUT) && (gameOver == false)) || ((gameOver == true) && (winTeam == false))) {

				glTexCoord2d(sRight,sBottom);	glVertex3i(RES*3, -RES*5, drawPos.y*RES);
				glTexCoord2d(sRight,sTop);	glVertex3i(RES*3, RES*7, drawPos.y*RES);
				glTexCoord2d(sLeft,sBottom);	glVertex3i(-RES*3, -RES*5, drawPos.y*RES);
				glTexCoord2d(sLeft,sTop);	glVertex3i(-RES*3,RES*7, drawPos.y*RES);
			} else {
				glTexCoord2d(sRight,sBottom);	glVertex3i(PLAYERWIDTH/2, -PLAYERHEIGHT/2, drawPos.y*RES);
				glTexCoord2d(sRight,sTop);	glVertex3i(PLAYERWIDTH/2, PLAYERHEIGHT/2, drawPos.y*RES);
				glTexCoord2d(sLeft,sBottom);	glVertex3i(-PLAYERWIDTH/2,-PLAYERHEIGHT/2, drawPos.y*RES);
				glTexCoord2d(sLeft,sTop);	glVertex3i(-PLAYERWIDTH/2,PLAYERHEIGHT/2, drawPos.y*RES);
			}
		glEnd();

		glDisable(GL_TEXTURE_2D);


		glPopMatrix();

}

void Player::clock(unsigned long gTime, unsigned int *numBallsLeft) {
		if(notPlaying == true) return;

		float dt;
		switch(drawState) {
			case DMOVE:
					dt = (gTime - (float)startTime) / ((float)nextStepTimeStamp - startTime);
					if(dt > 1.0) {
						dt = 1.0;
					}
					drawPos.x = startPos.x * (1.0-dt) + endPos.x*dt;
					drawPos.y = startPos.y * (1.0-dt) + endPos.y*dt;
					break;

			case DTHROUGH1:
			case DTHROUGH2:
					dt = (gTime - (float)startTime) / ((float)nextStepTimeStamp - startTime);
					if(dt > 1.0) {
						dt = 1.0;
						drawState = DSTAND;
					}

					if((dt > 0.5) && (drawState == DTHROUGH1)) {
						drawState = DTHROUGH2;
						// create ball
						Vector dir;
						dir.x = sin((angle)*3.14/180);
						dir.y = cos((angle)*3.14/180);
						Vector pos;
						switch(teamID) {
							case 0: pos.x = (drawPos.x*RES) + offsetX;
									pos.y = (drawPos.y*RES) + offsetY;
									break;
							case 1: pos.x = offsetX - (drawPos.y*RES);
									pos.y = offsetY + (drawPos.x*RES);
									break;
							case 2: pos.x = offsetX - (drawPos.x*RES);
									pos.y = offsetY - (drawPos.y*RES);
									break;
							case 3: pos.x = offsetX + (drawPos.y*RES);
									pos.y = offsetY - (drawPos.x*RES);
									break;
						};
 						teamBalls->push_back(Ball(teamID, id, this, dir, pos, strength, gTime));
					}

					break;

			default: break;
		};



		if(gTime > nextStepTimeStamp) {
			if(isHit == true) {
				notPlaying = true;
				drawState = DISOUT;
				return;
			}
			//drawState = DDEFFEND;
			//drawState = DSTAND;
			lineNumber++;
			if(lineNumber >= playerCode->size()) {
				//notPlaying = true;
				outofCode = true;
				lineNumber = playerCode->size()+1;
				return;
			}
			switch(playerCode->at(lineNumber).opCode) {
				case MOVE:
					nextStepTimeStamp = sqrt( (drawPos.x-playerCode->at(lineNumber).x)*(drawPos.x-playerCode->at(lineNumber).x) + (drawPos.y-playerCode->at(lineNumber).y)*(drawPos.y-playerCode->at(lineNumber).y)) / moveVelocity + gTime;
					drawState = DMOVE;
					startTime = gTime;
					startPos = drawPos;
					endPos.x = playerCode->at(lineNumber).x;
					endPos.y = playerCode->at(lineNumber).y;
					break;
				case THROUGH:
					if(*numBallsLeft > 0) {
						nextStepTimeStamp = gTime + throughTime;
						startTime = gTime;
						drawState = DTHROUGH1;
						*numBallsLeft = *numBallsLeft - 1;
					}
					break;

				case TURN:	drawState = DSTAND;
							angle = playerCode->at(lineNumber).angle + offsetA;
							while(angle < 0) {
								angle = angle + 360;
							}
							while(angle > 360) {
								angle = angle - 360;
							}
							break;

				case DEFFEND:
					drawState = DDEFFEND;
					nextStepTimeStamp = gTime +  playerCode->at(lineNumber).repeatCount;
					break;

				default: break;
			};
		}
}


void Player::getState(float *sRight, float *sLeft, float *sTop, float *sBottom, bool gameOver, bool winTeam){

	if(gameOver == true) {
		if(winTeam == true) {
			*sTop = 1.0f;
			*sBottom =0.0f;
			*sRight = 500/512.0f;
			*sLeft = 446/512.0f;
		} else {
			*sTop = 2.0f;
			*sBottom =30/64.0f;
			*sRight = 374/512.0f;
			*sLeft = 312/512.0f;
		}
		return;
	}

	switch(drawState) {

		case DDEFFEND:
		case DSTAND:
		case DMOVE:
		case DTHROUGH1:
			*sTop = 1.0f;
			*sBottom =0.0f;

			if(angle < 45) {
				//zone 1
				*sRight = 208/512.0f;
				*sLeft = 172/512.0f;
				return;
			} else {
				if(angle < 130) {
					//zone 2
					*sRight = 171/512.0f;
					*sLeft = 139/512.0f;
					return;
				} else {
					if(angle < 180) {
						//zone 3
						*sRight = 139/512.0f;
						*sLeft = 105/512.0f;
						return;
					} else {
						if(angle < 225) {
						//zone 4
							*sRight = 104/512.0f;
							*sLeft = 69/512.0f;
							return;
						} else {
							if(angle < 315) {
								// zone 5
								*sRight = 68/512.0f;
								*sLeft = 37/512.0f;
								return;
							} else {
								*sRight = 36/512.0f;
								*sLeft = 0.0f;
								return;
							}
						}
					}
				}
			}
			break;

		case DTHROUGH2:

			*sTop = 1.0f;
			*sBottom =0.0f;
			if(angle < 180) {
				*sRight = 307/512.0f;
				*sLeft = 260/512.0f;
			} else {
				*sRight = 256/512.0f;
				*sLeft = 210/512.0f;
			}
			return;
			break;

		case DISOUT:
			*sTop = 2.0f;
			*sBottom =30/64.0f;
			*sRight = 374/512.0f;
			*sLeft = 312/512.0f;
			return;
			break;


		case DHIT:
			*sTop = 1.0f;
			*sBottom = 0.0f;
			*sRight = 432/512.0f;
			*sLeft = 379/512.0f;
			return;
			break;


	};

}
